/* SPDX-License-Identifier: (LGPL-2.1 OR BSD-2-Clause) */

/* THIS FILE IS AUTOGENERATED! */
#ifndef __TEST_MAP_INIT_SKEL_H__
#define __TEST_MAP_INIT_SKEL_H__

#include <errno.h>
#include <stdlib.h>
#include <bpf/libbpf.h>

struct test_map_init {
	struct bpf_object_skeleton *skeleton;
	struct bpf_object *obj;
	struct {
		struct bpf_map *hashmap1;
		struct bpf_map *bss;
	} maps;
	struct {
		struct bpf_program *sysenter_getpgid;
	} progs;
	struct {
		struct bpf_link *sysenter_getpgid;
	} links;
	struct test_map_init__bss {
		__u64 inKey;
		__u64 inValue;
		__u32 inPid;
	} *bss;
};

static void
test_map_init__destroy(struct test_map_init *obj)
{
	if (!obj)
		return;
	if (obj->skeleton)
		bpf_object__destroy_skeleton(obj->skeleton);
	free(obj);
}

static inline int
test_map_init__create_skeleton(struct test_map_init *obj);

static inline struct test_map_init *
test_map_init__open_opts(const struct bpf_object_open_opts *opts)
{
	struct test_map_init *obj;
	int err;

	obj = (struct test_map_init *)calloc(1, sizeof(*obj));
	if (!obj) {
		errno = ENOMEM;
		return NULL;
	}

	err = test_map_init__create_skeleton(obj);
	err = err ?: bpf_object__open_skeleton(obj->skeleton, opts);
	if (err)
		goto err_out;

	return obj;
err_out:
	test_map_init__destroy(obj);
	errno = -err;
	return NULL;
}

static inline struct test_map_init *
test_map_init__open(void)
{
	return test_map_init__open_opts(NULL);
}

static inline int
test_map_init__load(struct test_map_init *obj)
{
	return bpf_object__load_skeleton(obj->skeleton);
}

static inline struct test_map_init *
test_map_init__open_and_load(void)
{
	struct test_map_init *obj;
	int err;

	obj = test_map_init__open();
	if (!obj)
		return NULL;
	err = test_map_init__load(obj);
	if (err) {
		test_map_init__destroy(obj);
		errno = -err;
		return NULL;
	}
	return obj;
}

static inline int
test_map_init__attach(struct test_map_init *obj)
{
	return bpf_object__attach_skeleton(obj->skeleton);
}

static inline void
test_map_init__detach(struct test_map_init *obj)
{
	return bpf_object__detach_skeleton(obj->skeleton);
}

static inline int
test_map_init__create_skeleton(struct test_map_init *obj)
{
	struct bpf_object_skeleton *s;

	s = (struct bpf_object_skeleton *)calloc(1, sizeof(*s));
	if (!s)
		goto err;
	obj->skeleton = s;

	s->sz = sizeof(*s);
	s->name = "test_map_init";
	s->obj = &obj->obj;

	/* maps */
	s->map_cnt = 2;
	s->map_skel_sz = sizeof(*s->maps);
	s->maps = (struct bpf_map_skeleton *)calloc(s->map_cnt, s->map_skel_sz);
	if (!s->maps)
		goto err;

	s->maps[0].name = "hashmap1";
	s->maps[0].map = &obj->maps.hashmap1;

	s->maps[1].name = "test_map.bss";
	s->maps[1].map = &obj->maps.bss;
	s->maps[1].mmaped = (void **)&obj->bss;

	/* programs */
	s->prog_cnt = 1;
	s->prog_skel_sz = sizeof(*s->progs);
	s->progs = (struct bpf_prog_skeleton *)calloc(s->prog_cnt, s->prog_skel_sz);
	if (!s->progs)
		goto err;

	s->progs[0].name = "sysenter_getpgid";
	s->progs[0].prog = &obj->progs.sysenter_getpgid;
	s->progs[0].link = &obj->links.sysenter_getpgid;

	s->data_sz = 2544;
	s->data = (void *)"\
\x7f\x45\x4c\x46\x02\x01\x01\0\0\0\0\0\0\0\0\0\x01\0\xf7\0\x01\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x70\x07\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\x40\0\x0a\0\
\x01\0\0\x2e\x73\x74\x72\x74\x61\x62\0\x2e\x73\x79\x6d\x74\x61\x62\0\x74\x70\
\x2f\x73\x79\x73\x63\x61\x6c\x6c\x73\x2f\x73\x79\x73\x5f\x65\x6e\x74\x65\x72\
\x5f\x67\x65\x74\x70\x67\x69\x64\0\x2e\x62\x73\x73\0\x2e\x6d\x61\x70\x73\0\x6c\
\x69\x63\x65\x6e\x73\x65\0\x74\x65\x73\x74\x5f\x6d\x61\x70\x5f\x69\x6e\x69\x74\
\x2e\x63\0\x4c\x42\x42\x30\x5f\x32\0\x73\x79\x73\x65\x6e\x74\x65\x72\x5f\x67\
\x65\x74\x70\x67\x69\x64\0\x69\x6e\x50\x69\x64\0\x68\x61\x73\x68\x6d\x61\x70\
\x31\0\x69\x6e\x4b\x65\x79\0\x69\x6e\x56\x61\x6c\x75\x65\0\x5f\x6c\x69\x63\x65\
\x6e\x73\x65\0\x2e\x72\x65\x6c\x74\x70\x2f\x73\x79\x73\x63\x61\x6c\x6c\x73\x2f\
\x73\x79\x73\x5f\x65\x6e\x74\x65\x72\x5f\x67\x65\x74\x70\x67\x69\x64\0\x2e\x42\
\x54\x46\0\x2e\x42\x54\x46\x2e\x65\x78\x74\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\x42\0\0\0\x04\0\xf1\xff\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\x03\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x52\0\0\0\0\0\x03\0\x70\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\x59\0\0\0\x12\0\x03\0\0\0\0\0\0\0\0\0\x80\0\0\0\0\0\0\0\
\x6a\0\0\0\x11\0\x04\0\x10\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\x70\0\0\0\x11\0\x05\
\0\0\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x79\0\0\0\x11\0\x04\0\0\0\0\0\0\0\0\0\x08\
\0\0\0\0\0\0\0\x7f\0\0\0\x11\0\x04\0\x08\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\x87\0\
\0\0\x11\0\x06\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\x85\0\0\0\x0e\0\0\0\x77\0\0\
\0\x20\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x61\x11\0\0\0\0\0\0\x5d\x01\
\x08\0\0\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x18\x02\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\x18\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xb7\x04\0\0\x01\0\0\0\x85\0\0\0\
\x02\0\0\0\xb7\0\0\0\0\0\0\0\x95\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x47\x50\x4c\0\0\0\0\0\x10\0\0\0\0\0\0\0\x01\0\
\0\0\x05\0\0\0\x30\0\0\0\0\0\0\0\x01\0\0\0\x06\0\0\0\x40\0\0\0\0\0\0\0\x01\0\0\
\0\x07\0\0\0\x50\0\0\0\0\0\0\0\x01\0\0\0\x08\0\0\0\x9f\xeb\x01\0\x18\0\0\0\0\0\
\0\0\xf8\x01\0\0\xf8\x01\0\0\xe2\x01\0\0\0\0\0\0\0\0\0\x02\x03\0\0\0\x01\0\0\0\
\0\0\0\x01\x04\0\0\0\x20\0\0\x01\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\
\x05\0\0\0\x05\0\0\0\0\0\0\x01\x04\0\0\0\x20\0\0\0\0\0\0\0\0\0\0\x02\x06\0\0\0\
\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x02\0\0\0\0\0\0\0\0\0\0\x02\x08\
\0\0\0\x19\0\0\0\0\0\0\x08\x09\0\0\0\x1f\0\0\0\0\0\0\x01\x08\0\0\0\x40\0\0\0\0\
\0\0\0\x04\0\0\x04\x20\0\0\0\x36\0\0\0\x01\0\0\0\0\0\0\0\x3b\0\0\0\x05\0\0\0\
\x40\0\0\0\x47\0\0\0\x07\0\0\0\x80\0\0\0\x4b\0\0\0\x07\0\0\0\xc0\0\0\0\x51\0\0\
\0\0\0\0\x0e\x0a\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x0d\0\0\0\0\0\0\0\0\0\0\x0a\
\0\0\0\0\0\0\0\0\x01\0\0\x0d\x02\0\0\0\x5a\0\0\0\x0c\0\0\0\x5e\0\0\0\x01\0\0\
\x0c\x0e\0\0\0\x6f\0\0\0\0\0\0\x0e\x08\0\0\0\x01\0\0\0\x75\0\0\0\0\0\0\x0e\x08\
\0\0\0\x01\0\0\0\x7d\0\0\0\0\0\0\x08\x13\0\0\0\x83\0\0\0\0\0\0\x01\x04\0\0\0\
\x20\0\0\0\x90\0\0\0\0\0\0\x0e\x12\0\0\0\x01\0\0\0\x96\0\0\0\0\0\0\x01\x01\0\0\
\0\x08\0\0\x01\0\0\0\0\0\0\0\x03\0\0\0\0\x15\0\0\0\x04\0\0\0\x04\0\0\0\x9b\0\0\
\0\0\0\0\x0e\x16\0\0\0\x01\0\0\0\xb1\x01\0\0\x03\0\0\x0f\x14\0\0\0\x10\0\0\0\0\
\0\0\0\x08\0\0\0\x11\0\0\0\x08\0\0\0\x08\0\0\0\x14\0\0\0\x10\0\0\0\x04\0\0\0\
\xb6\x01\0\0\x01\0\0\x0f\x20\0\0\0\x0b\0\0\0\0\0\0\0\x20\0\0\0\xbc\x01\0\0\x01\
\0\0\x0f\x04\0\0\0\x17\0\0\0\0\0\0\0\x04\0\0\0\0\x69\x6e\x74\0\x5f\x5f\x41\x52\
\x52\x41\x59\x5f\x53\x49\x5a\x45\x5f\x54\x59\x50\x45\x5f\x5f\0\x5f\x5f\x75\x36\
\x34\0\x6c\x6f\x6e\x67\x20\x6c\x6f\x6e\x67\x20\x75\x6e\x73\x69\x67\x6e\x65\x64\
\x20\x69\x6e\x74\0\x74\x79\x70\x65\0\x6d\x61\x78\x5f\x65\x6e\x74\x72\x69\x65\
\x73\0\x6b\x65\x79\0\x76\x61\x6c\x75\x65\0\x68\x61\x73\x68\x6d\x61\x70\x31\0\
\x63\x74\x78\0\x73\x79\x73\x65\x6e\x74\x65\x72\x5f\x67\x65\x74\x70\x67\x69\x64\
\0\x69\x6e\x4b\x65\x79\0\x69\x6e\x56\x61\x6c\x75\x65\0\x5f\x5f\x75\x33\x32\0\
\x75\x6e\x73\x69\x67\x6e\x65\x64\x20\x69\x6e\x74\0\x69\x6e\x50\x69\x64\0\x63\
\x68\x61\x72\0\x5f\x6c\x69\x63\x65\x6e\x73\x65\0\x2f\x62\x75\x69\x6c\x64\x64\
\x69\x72\x2f\x62\x75\x69\x6c\x64\x2f\x42\x55\x49\x4c\x44\x2f\x6b\x65\x72\x6e\
\x65\x6c\x2d\x35\x2e\x31\x34\x2e\x30\x2d\x37\x31\x2e\x65\x6c\x39\x2f\x6c\x69\
\x6e\x75\x78\x2d\x35\x2e\x31\x34\x2e\x30\x2d\x37\x31\x2e\x65\x6c\x39\x2e\x61\
\x61\x72\x63\x68\x36\x34\x2f\x74\x6f\x6f\x6c\x73\x2f\x74\x65\x73\x74\x69\x6e\
\x67\x2f\x73\x65\x6c\x66\x74\x65\x73\x74\x73\x2f\x62\x70\x66\x2f\x70\x72\x6f\
\x67\x73\x2f\x74\x65\x73\x74\x5f\x6d\x61\x70\x5f\x69\x6e\x69\x74\x2e\x63\0\x09\
\x69\x6e\x74\x20\x63\x75\x72\x5f\x70\x69\x64\x20\x3d\x20\x62\x70\x66\x5f\x67\
\x65\x74\x5f\x63\x75\x72\x72\x65\x6e\x74\x5f\x70\x69\x64\x5f\x74\x67\x69\x64\
\x28\x29\x20\x3e\x3e\x20\x33\x32\x3b\0\x09\x69\x66\x20\x28\x63\x75\x72\x5f\x70\
\x69\x64\x20\x3d\x3d\x20\x69\x6e\x50\x69\x64\x29\0\x09\x09\x62\x70\x66\x5f\x6d\
\x61\x70\x5f\x75\x70\x64\x61\x74\x65\x5f\x65\x6c\x65\x6d\x28\x26\x68\x61\x73\
\x68\x6d\x61\x70\x31\x2c\x20\x26\x69\x6e\x4b\x65\x79\x2c\x20\x26\x69\x6e\x56\
\x61\x6c\x75\x65\x2c\x20\x42\x50\x46\x5f\x4e\x4f\x45\x58\x49\x53\x54\x29\x3b\0\
\x09\x72\x65\x74\x75\x72\x6e\x20\x30\x3b\0\x2e\x62\x73\x73\0\x2e\x6d\x61\x70\
\x73\0\x6c\x69\x63\x65\x6e\x73\x65\0\x74\x70\x2f\x73\x79\x73\x63\x61\x6c\x6c\
\x73\x2f\x73\x79\x73\x5f\x65\x6e\x74\x65\x72\x5f\x67\x65\x74\x70\x67\x69\x64\0\
\0\0\0\0\0\0\x9f\xeb\x01\0\x20\0\0\0\0\0\0\0\x14\0\0\0\x14\0\0\0\x6c\0\0\0\x80\
\0\0\0\0\0\0\0\x08\0\0\0\xc4\x01\0\0\x01\0\0\0\0\0\0\0\x0f\0\0\0\x10\0\0\0\xc4\
\x01\0\0\x06\0\0\0\0\0\0\0\xa4\0\0\0\x1d\x01\0\0\x10\x64\0\0\x08\0\0\0\xa4\0\0\
\0\x1d\x01\0\0\x2b\x64\0\0\x10\0\0\0\xa4\0\0\0\x4e\x01\0\0\x11\x6c\0\0\x28\0\0\
\0\xa4\0\0\0\x4e\x01\0\0\x06\x6c\0\0\x30\0\0\0\xa4\0\0\0\x65\x01\0\0\x03\x70\0\
\0\x70\0\0\0\xa4\0\0\0\xa6\x01\0\0\x02\x78\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\x01\0\0\0\x03\0\0\0\x20\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x40\0\
\0\0\0\0\0\0\xc0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\x09\0\0\0\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\xf0\0\
\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\x18\0\0\0\0\0\0\0\x11\0\0\0\
\x01\0\0\0\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xf0\x01\0\0\0\0\0\0\x80\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x2f\0\0\0\x08\0\0\0\x03\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x70\x02\0\0\0\0\0\0\x14\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x34\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\x70\x02\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\x3a\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\x90\x02\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\x90\0\0\0\x09\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x98\x02\0\0\0\
\0\0\0\x40\0\0\0\0\0\0\0\x02\0\0\0\x03\0\0\0\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\
\0\xb2\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xd8\x02\0\0\0\0\0\0\xf2\
\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xb7\0\0\0\
\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xd0\x06\0\0\0\0\0\0\xa0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";

	return 0;
err:
	bpf_object__destroy_skeleton(s);
	return -ENOMEM;
}

#endif /* __TEST_MAP_INIT_SKEL_H__ */
